@@ -1,7 +1,7 @@
 // samplv1_jack.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2022, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2023, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -679,6 +679,12 @@ void samplv1_jack::updateLoopZero (void)
 }
 
 
+void samplv1_jack::updateLoopRelease (void)
+{
+	// nothing to do here...
+}
+
+
 void samplv1_jack::updateTuning (void)
 {
 	samplv1::resetTuning();
@@ -723,6 +729,8 @@ void samplv1_jack::shutdown_close (void)
 #endif
 #endif
 
+#include <QTimer>
+
 #ifdef CONFIG_NSM
 #include "samplv1_nsm.h"
 #endif
@@ -1049,6 +1057,9 @@ bool samplv1_jack_application::setup (void)
 	if (m_pWidget)
 		m_pWidget->show();
 
+	// Start watchdog timer...
+	watchdog_start();
+
 	return true;
 }
 
@@ -1091,19 +1102,23 @@ void samplv1_jack_application::openSession (void)
 	if (!dir.exists())
 		dir.mkpath(path_name);
 
+	bool bOpen = false;
+
 	QFileInfo fi(path_name, "session." SAMPLV1_TITLE);
 	if (!fi.exists())
 		fi.setFile(path_name, display_name + '.' + SAMPLV1_TITLE);
 	if (fi.exists()) {
 		const QString& sFilename = fi.absoluteFilePath();
 		if (m_pWidget) {
-			m_pWidget->loadPreset(sFilename);
+			bOpen = m_pWidget->loadPreset(sFilename);
 		} else {
-			samplv1_param::loadPreset(m_pSampl, sFilename);
+			bOpen = samplv1_param::loadPreset(m_pSampl, sFilename);
 		}
 	}
 
-	m_pNsmClient->open_reply();
+	m_pNsmClient->open_reply(bOpen
+		? samplv1_nsm::ERR_OK
+		: samplv1_nsm::ERR_GENERAL);
 	m_pNsmClient->dirty(false);
 
 	if (m_pWidget)
@@ -1131,9 +1146,12 @@ void samplv1_jack_application::saveSession (void)
 //	const QFileInfo fi(path_name, display_name + '.' + SAMPLV1_TITLE);
 	const QFileInfo fi(path_name, "session." SAMPLV1_TITLE);
 
-	samplv1_param::savePreset(m_pSampl, fi.absoluteFilePath(), true);
+	const bool bSave
+		= samplv1_param::savePreset(m_pSampl, fi.absoluteFilePath(), true);
 
-	m_pNsmClient->save_reply();
+	m_pNsmClient->save_reply(bSave
+		? samplv1_nsm::ERR_OK
+		: samplv1_nsm::ERR_GENERAL);
 	m_pNsmClient->dirty(false);
 }
 
@@ -1199,6 +1217,29 @@ void samplv1_jack_application::handle_sigterm (void)
 #endif	// HAVE_SIGNAL_H
 
 
+// Simple watchdog (3 minute cycle).
+void samplv1_jack_application::watchdog_slot (void)
+{
+	if (m_pSampl && m_pSampl->client() == nullptr) {
+		const QByteArray aClientName
+			= m_sClientName.toLocal8Bit();
+		const char *client_name
+			= aClientName.constData();
+		m_pSampl->open(client_name);
+		m_pSampl->activate();
+	}
+
+	watchdog_start();
+}
+
+
+void samplv1_jack_application::watchdog_start (void)
+{
+	if (g_pInstance)
+		QTimer::singleShot(60000, this, SLOT(watchdog_slot()));
+}
+
+
 // JACK shutdown handlers.
 void samplv1_jack_application::shutdown (void)
 {
@@ -1208,16 +1249,18 @@ void samplv1_jack_application::shutdown (void)
 
 void samplv1_jack_application::shutdown_slot (void)
 {
-	bool bQuit = true;
-
 	if (m_pSampl)
 		m_pSampl->shutdown_close();
 
+#if 0//Don't quit anymore; let watchdog handle auto re-activation...
+	bool bQuit = true;
+
 	if (m_pWidget)
 		bQuit = m_pWidget->queryClose();
 
 	if (m_pApp && bQuit)
 		m_pApp->quit();
+#endif
 }
 
 
