@@ -1,7 +1,7 @@
 // samplv1widget.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2022, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2023, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -571,6 +571,9 @@ samplv1widget::samplv1widget ( QWidget *pParent )
 	QObject::connect(m_ui.Gen1LoopZeroCheckBox,
 		SIGNAL(valueChanged(float)),
 		SLOT(loopZeroChanged()));
+	QObject::connect(m_ui.Gen1LoopReleaseCheckBox,
+		SIGNAL(valueChanged(float)),
+		SLOT(loopReleaseChanged()));
 
 	QObject::connect(m_ui.Gen1OffsetStartSpinBox,
 		SIGNAL(customContextMenuRequested(const QPoint&)),
@@ -1108,7 +1111,7 @@ void samplv1widget::newPreset (void)
 
 
 // Preset file I/O slots.
-void samplv1widget::loadPreset ( const QString& sFilename )
+bool samplv1widget::loadPreset ( const QString& sFilename )
 {
 #ifdef CONFIG_DEBUG
 	qDebug("samplv1widget::loadPreset(\"%s\")", sFilename.toUtf8().constData());
@@ -1119,29 +1122,41 @@ void samplv1widget::loadPreset ( const QString& sFilename )
 	resetParamKnobs();
 	resetParamValues();
 
+	bool bLoad = false;
+
 	samplv1_ui *pSamplUi = ui_instance();
 	if (pSamplUi)
-		pSamplUi->loadPreset(sFilename);
+		bLoad = pSamplUi->loadPreset(sFilename);
 
-	updateLoadPreset(QFileInfo(sFilename).completeBaseName());
+	if (bLoad)
+		updateLoadPreset(QFileInfo(sFilename).completeBaseName());
+	else
+		updateDirtyPreset(true);
+
+	return bLoad;
 }
 
 
-void samplv1widget::savePreset ( const QString& sFilename )
+bool samplv1widget::savePreset ( const QString& sFilename )
 {
 #ifdef CONFIG_DEBUG
 	qDebug("samplv1widget::savePreset(\"%s\")", sFilename.toUtf8().constData());
 #endif
 
+	bool bSave = false;
+
 	samplv1_ui *pSamplUi = ui_instance();
 	if (pSamplUi)
-		pSamplUi->savePreset(sFilename);
+		bSave = pSamplUi->savePreset(sFilename);
 
-	const QString& sPreset
-		= QFileInfo(sFilename).completeBaseName();
+	if (bSave) {
+		const QString& sPreset
+			= QFileInfo(sFilename).completeBaseName();
+		m_ui.StatusBar->showMessage(tr("Save preset: %1").arg(sPreset), 5000);
+	}
+	updateDirtyPreset(!bSave);
 
-	m_ui.StatusBar->showMessage(tr("Save preset: %1").arg(sPreset), 5000);
-	updateDirtyPreset(false);
+	return bSave;
 }
 
 
@@ -1382,6 +1397,25 @@ void samplv1widget::loopZeroChanged (void)
 }
 
 
+// Loop end-release change.
+void samplv1widget::loopReleaseChanged (void)
+{
+	if (m_iUpdate > 0)
+		return;
+
+	++m_iUpdate;
+	samplv1_ui *pSamplUi = ui_instance();
+	if (pSamplUi) {
+		const bool bLoopRelease = (m_ui.Gen1LoopReleaseCheckBox->value() > 0.5f);
+		pSamplUi->setLoopRelease(bLoopRelease);
+		m_ui.StatusBar->showMessage(tr("Loop end-release: %1")
+			.arg(bLoopRelease ? tr("On") : tr("Off")), 5000);
+		updateDirtyPreset(true);
+	}
+	--m_iUpdate;
+}
+
+
 // Offset points changed (from UI).
 void samplv1widget::offsetRangeChanged (void)
 {
@@ -1430,6 +1464,7 @@ void samplv1widget::updateOffsetLoop ( samplv1_sample *pSample, bool bDirty )
 		const uint32_t iLoopEnd   = pSample->loopEnd();
 		const uint32_t iLoopFade  = pSample->loopCrossFade();
 		const bool     bLoopZero  = pSample->isLoopZeroCrossing();
+		const bool     bLoopRelease = pSample->isLoopEndRelease();
 		const uint32_t nframes    = pSample->length();
 		const float    srate      = pSample->sampleRate();
 		m_ui.Gen1OctavesComboBox->setCurrentIndex(iOctaves);
@@ -1465,6 +1500,8 @@ void samplv1widget::updateOffsetLoop ( samplv1_sample *pSample, bool bDirty )
 		m_ui.Gen1LoopFadeSpinBox->setValue(m_iLoopFade);
 		m_ui.Gen1LoopZeroCheckBox->setValue(bLoopZero ? 1.0f : 0.0f);
 		m_ui.Gen1LoopZeroCheckBox->setEnabled(bLoop);
+		m_ui.Gen1LoopReleaseCheckBox->setValue(bLoopRelease ? 1.0f : 0.0f);
+		m_ui.Gen1LoopReleaseCheckBox->setEnabled(bLoop);
 		m_ui.Gen1Sample->setOffsetStart(iOffsetStart);
 		m_ui.Gen1Sample->setOffsetEnd(iOffsetEnd);
 		m_ui.Gen1Sample->setOffset(bOffset);
@@ -1520,6 +1557,7 @@ void samplv1widget::updateOffsetLoop ( samplv1_sample *pSample, bool bDirty )
 		m_ui.Gen1LoopFadeSpinBox->setMaximum(0);
 		m_ui.Gen1LoopFadeSpinBox->setValue(0);
 		m_ui.Gen1LoopZeroCheckBox->setEnabled(false);
+		m_ui.Gen1LoopReleaseCheckBox->setEnabled(false);
 		m_ui.Gen1Sample->setOffsetStart(0);
 		m_ui.Gen1Sample->setOffsetEnd(0);
 		m_ui.Gen1Sample->setOffset(false);
@@ -1707,10 +1745,8 @@ void samplv1widget::midiInLedTimeout (void)
 void samplv1widget::helpConfigure (void)
 {
 	samplv1_ui *pSamplUi = ui_instance();
-	if (pSamplUi == nullptr)
-		return;
-
-	samplv1widget_config(pSamplUi, this).exec();
+	if (pSamplUi)
+		samplv1widget_config(this, pSamplUi).exec();
 }
 
 
@@ -1737,15 +1773,15 @@ void samplv1widget::helpAbout (void)
 	list << tr("LV2 plug-in build disabled.");
 #endif
 
-	QString sText = "<p>\n";
-	sText += "<b>" SAMPLV1_TITLE "</b> - " + tr(SAMPLV1_SUBTITLE) + "<br />\n";
+	QString sText = "<h1>" SAMPLV1_TITLE "</h1>\n";
+	sText += "<p>" + tr(SAMPLV1_SUBTITLE) + "<br />\n";
 	sText += "<br />\n";
 	sText += tr("Version") + ": <b>" CONFIG_BUILD_VERSION "</b><br />\n";
 //	sText += "<small>" + tr("Build") + ": " CONFIG_BUILD_DATE "</small><br />\n";
 	if (!list.isEmpty()) {
 		sText += "<small><font color=\"red\">";
 		sText += list.join("<br />\n");
-		sText += "</font></small><br />\n";
+		sText += "</font></small>\n";
 	}
 	sText += "<br />\n";
 	sText += tr("Using: Qt %1").arg(qVersion());
