@@ -1,7 +1,7 @@
 // samplv1widget_config.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2022, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2023, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -57,7 +57,7 @@ static const char *g_pszDefName = QT_TRANSLATE_NOOP("samplv1widget_config", "(de
 
 // ctor.
 samplv1widget_config::samplv1widget_config (
-	samplv1_ui *pSamplUi, QWidget *pParent )
+	samplv1widget *pParent, samplv1_ui *pSamplUi )
 	: QDialog(pParent), p_ui(new Ui::samplv1widget_config), m_ui(*p_ui),
 		m_pSamplUi(pSamplUi)
 {
@@ -90,6 +90,11 @@ samplv1widget_config::samplv1widget_config (
 	m_iDirtyPrograms = 0;
 	m_iDirtyOptions  = 0;
 
+	// Whether presets exist...
+	m_bPresets = false;
+
+	m_iLoadPreset = 0;
+
 	// Setup options...
 	samplv1_config *pConfig = samplv1_config::getInstance();
 	if (pConfig && m_pSamplUi) {
@@ -114,9 +119,10 @@ samplv1widget_config::samplv1widget_config (
 		// Load programs database...
 		samplv1_programs *pPrograms = m_pSamplUi->programs();
 		if (pPrograms) {
+			m_bPresets = !pConfig->presetList().isEmpty();
 			m_ui.ProgramsTreeWidget->loadPrograms(pPrograms);
-			m_ui.ProgramsEnabledCheckBox->setEnabled(bPlugin);
-			m_ui.ProgramsPreviewCheckBox->setEnabled(!bPlugin);
+			m_ui.ProgramsEnabledCheckBox->setEnabled(bPlugin && m_bPresets);
+			m_ui.ProgramsPreviewCheckBox->setEnabled(!bPlugin && m_bPresets);
 			m_ui.ProgramsEnabledCheckBox->setChecked(pPrograms->enabled());
 		}
 		// Initialize conveniency options...
@@ -463,11 +469,8 @@ void samplv1widget_config::programsChanged (void)
 
 void samplv1widget_config::programsActivated (void)
 {
-	if (m_pSamplUi) {
-		samplv1_programs *pPrograms = m_pSamplUi->programs();
-		if (m_ui.ProgramsPreviewCheckBox->isChecked() && pPrograms)
-			m_ui.ProgramsTreeWidget->selectProgram(pPrograms);
-	}
+	if (m_ui.ProgramsPreviewCheckBox->isChecked())
+		loadPreset(m_ui.ProgramsTreeWidget->currentProgramName());
 
 	stabilize();
 }
@@ -660,7 +663,7 @@ void samplv1widget_config::stabilize (void)
 	m_ui.ControlsDeleteToolButton->setEnabled(bEnabled);
 
 	pItem = m_ui.ProgramsTreeWidget->currentItem();
-	bEnabled = (m_pSamplUi && m_pSamplUi->programs() != nullptr);
+	bEnabled = (m_pSamplUi && m_pSamplUi->programs() != nullptr && m_bPresets);
 	m_ui.ProgramsPreviewCheckBox->setEnabled(
 		bEnabled && m_ui.ProgramsEnabledCheckBox->isChecked());
 	m_ui.ProgramsAddBankToolButton->setEnabled(bEnabled);
@@ -687,7 +690,8 @@ void samplv1widget_config::stabilize (void)
 		= (m_iDirtyTuning   > 0
 		|| m_iDirtyControls > 0
 		|| m_iDirtyPrograms > 0
-		|| m_iDirtyOptions  > 0);
+		|| m_iDirtyOptions  > 0
+		|| m_iLoadPreset    > 0);
 	m_ui.DialogButtonBox->button(QDialogButtonBox::Ok)->setEnabled(bValid);
 }
 
@@ -828,7 +832,8 @@ void samplv1widget_config::reject (void)
 	if (m_iDirtyTuning   > 0 ||
 		m_iDirtyControls > 0 ||
 		m_iDirtyPrograms > 0 ||
-		m_iDirtyOptions  > 0) {
+		m_iDirtyOptions  > 0 ||
+		m_iLoadPreset    > 0) {
 		QMessageBox::StandardButtons buttons
 			= QMessageBox::Discard | QMessageBox::Cancel;
 		if (m_ui.DialogButtonBox->button(QDialogButtonBox::Ok)->isEnabled())
@@ -843,11 +848,14 @@ void samplv1widget_config::reject (void)
 			return;
 		case QMessageBox::Discard:
 			break;
-		default:    // Cancel.
+		default: // Cancel.
 			bReject = false;
 		}
 	}
 
+	if (bReject && m_iLoadPreset > 0)
+		loadPreset(m_sSavePreset);
+
 	if (bReject)
 		QDialog::reject();
 }
@@ -1012,4 +1020,26 @@ QString samplv1widget_config::comboBoxCurrentItem ( QComboBox *pComboBox )
 }
 
 
+// Programs/preset preview stuff...
+void samplv1widget_config::loadPreset ( const QString& sPreset )
+{
+	samplv1_config *pConfig = samplv1_config::getInstance();
+	if (pConfig == nullptr)
+		return;
+
+	const QString& sPresetFile
+		= pConfig->presetFile(sPreset);
+	if (sPresetFile.isEmpty())
+		return;
+
+	samplv1widget *pParentWidget
+		= qobject_cast<samplv1widget *> (parentWidget());
+	if (pParentWidget && pParentWidget->loadPreset(sPresetFile)) {
+		if (++m_iLoadPreset == 1)
+			m_sSavePreset = pConfig->sPreset;
+		pConfig->sPreset = sPreset;
+	}
+}
+
+
 // end of samplv1widget_config.cpp
